function [posRand,triRank]=customRPG(tri,opp,pos,posEstim,...
  cost,costEstim,morph,around,rnk,reTriang, allPos,...
  toQuantis,quantis)
%{
  Inputs
  ------
  tri					Triangles defining the triangulation
  opp					Flag saying we are looking for a minimum or a maximum
  pos					Known positions (i.e. whose value is sure)
  posEstim		Estimated positions
  cost				Values of the known positions
  costEstim		Values of the estimated positions
  morph				Parameter to "distort" the distribution. Default value 1
  around			Option to look "around" a position
  rnk					Option to use the values or their ranks
  reTriang		Flag for complete re-triangulation or not
  allPos			true => Takes all positions into account (known and estimated)
  						false => only estimated positions
  toQuantis		flag (discretisation or not)
  quantis			discretisation step
  
  Output
  -----
  posRand			New sampled position
  triRank			rank of the triangle that contains it. Maybe 0 if the
  						triangulation does not "cover" all the search space.
 
  
  Principles of the custom-made RPG:
   1) Triangulation on the known points (Delaunay or simplified)
   2) For each  'triangle ', evaluates a representative, for example  its center.
    So, we define a list (position_i, estimated_cost_i)
   3) Define the concatenated interval I = [0,known_cost_1[ U [known_cost_2, known_cost 3] ...
        U [0,estimated_cost_1[ U [estimated_cost_1, estimated_cost_2[, ...
   4) Draw at random (uniform distribution) a point in I
   5) Find the corresponding index i
   
   Note: 3, 4 and 5 can be done by directly using a multinomial
   				distribution
   
   6a) If this is the one of an estimated position:
        really evaluate position_i. It is a new known point.
   6b) If this is the one of a known position, draw a point at random  'around it '.
       See aroundPos.m for different methods (parameter "around")
  
  %}
  
  [nbKnownPos,D]=size(pos); % Number of known positions, and dimension D
  nbEstimPos=length(costEstim); % Number of estimated positions
  if allPos
  	nbPos=nbEstimPos+nbKnownPos; % Total number of positions.
  	% Considers all values (known and estimated)
  val=[costEstim cost];
  else
  nbPos=nbEstimPos;
  val=costEstim;
  end
  
  if rnk==1   % Uses ranks instead values
    [val,rnkVal]=sort(val);
  end
  
  if opp==1   % Looking for a minimum
    val=min(val)+ max(val) -val;
  end
  
  % Ensures that the interval is [0, ...]
  val=val-min(val);
  
  % ---------------------------------------Distorts the distribution, if morph#1
  % to speed up or to slow down the convergence
  if morph~=1
    val=distort(val,morph);
  end
  
  %------------------------------------------------------ Defines a new position
  % The first step is to select a position, either a known one or
  % an estimated one, randomly, according to the distribution
  
  
%{
  valSum=cumsum(val);
  if valSum==0 % All positions are on a plateau, we choose one at random
    n=randi(nbPos);
  else % Draws a random number in [0 max(val)], uniform distribution
    r=valSum(nbPos)*rand;
    
    % Finds its rank in valSum
    n=dSearch(r,[0 valSum]);
  end
%}
	sumVal=sum(val);
	if sumVal>0
		r=mnrnd(1,val/sum(val)); % Multinomial distribution
			n=find(r);
	else
		  n=randi(nbPos);
	end 

  if rnk==1 % Option:  uses the real rank in [posEstim ; pos]
    % Remember that val has been sorted by increasing order
    n=rnkVal(n);
  end
  
  if n<=nbEstimPos % In this case, an estimated position has been selected.
    % Takes its position
    posRand=posEstim(n,:);
    triRank=n; % Is inside this facet, by definition (usually the center)

  else % In this case, an already known position has been selected.
    % Samples  'around ' it
    nn=n-nbEstimPos;
    [posRand,triRank]=aroundPos(nn,pos,posEstim, tri, around);
  
  end %if n<=nbEstimPos
  
   % Takes quantisation (discretisation) into account, if any
      if toQuantis
        posRand=quantify(posRand, quantis);
      end
end

